@@ -11,10 +11,10 @@ from io import BytesIO
 from itertools import zip_longest
 from pathlib import Path
 from shutil import copyfileobj
-from tempfile import NamedTemporaryFile
-from zlib import decompress, error as ZlibError
+from zlib import decompress
+from zlib import error as ZlibError
 
-from .. import Array, Dictionary, Name, Object, PdfError, Stream
+from .. import Array, Dictionary, Name, Object, PdfError, Stream, StreamDecodeLevel
 
 
 class DependencyError(Exception):
@@ -25,6 +25,10 @@ class UnsupportedImageTypeError(Exception):
     pass
 
 
+class NotExtractableError(Exception):
+    pass
+
+
 def array_str(value):
     if isinstance(value, (list, Array)):
         return [str(item) for item in value]
@@ -217,13 +221,12 @@ class PdfImageBase(ABC):
 
         if not self.indexed:
             return None
-        _idx, base, hival, lookup = None, None, None, None
         try:
-            _idx, base, hival, lookup = self._colorspaces
+            _idx, base, _hival, lookup = self._colorspaces
         except ValueError as e:
             raise ValueError('Not sure how to interpret this palette') from e
         base = str(base)
-        hival = int(hival)
+        _hival = int(_hival)
         lookup = bytes(lookup)
         if not base in self.SIMPLE_COLORSPACES:
             raise NotImplementedError("not sure how to interpret this palette")
@@ -393,7 +396,7 @@ class PdfImage(PdfImageBase):
             stream.write(data)
             return '.jpg'
 
-        raise UnsupportedImageTypeError()
+        raise NotExtractableError()
 
     def _extract_transcoded(self):
         from PIL import Image
@@ -424,8 +427,18 @@ class PdfImage(PdfImageBase):
             im = Image.frombytes('1', self.size, data)
 
             base_mode, palette = self.palette
-            if not (palette == b'\x00\x00\x00\xff\xff\xff' or palette == b'\x00\xff'):
-                raise NotImplementedError('monochrome image with nontrivial palette')
+            if base_mode == 'RGB' and palette != b'\x00\x00\x00\xff\xff\xff':
+                im = im.convert('P')
+                im.putpalette(palette, rawmode=base_mode)
+                gp = im.getpalette()
+                gp[765:768] = gp[3:6]  # work around Pillow bug
+                im.putpalette(gp)
+            elif base_mode == 'L' and palette != b'\x00\xff':
+                im = im.convert('P')
+                im.putpalette(palette, rawmode=base_mode)
+                gp = im.getpalette()
+                gp[255] = gp[1]  # work around Pillow bug
+                im.putpalette(gp)
 
         if self.colorspace == '/ICCBased':
             im.info['icc_profile'] = self.icc.tobytes()
@@ -454,13 +467,18 @@ class PdfImage(PdfImageBase):
 
         try:
             return self._extract_direct(stream=stream)
-        except UnsupportedImageTypeError:
+        except NotExtractableError:
             pass
 
-        im = self._extract_transcoded()
-        if im:
-            im.save(stream, format='png')
-            return '.png'
+        try:
+            im = self._extract_transcoded()
+            if im:
+                im.save(stream, format='png')
+                return '.png'
+        except PdfError as e:
+            if 'getStreamData called on unfilterable stream' in str(e):
+                raise UnsupportedImageTypeError(repr(self)) from e
+            raise
 
         raise UnsupportedImageTypeError(repr(self))
 
@@ -514,11 +532,11 @@ class PdfImage(PdfImageBase):
 
     def read_bytes(self):
         """Decompress this image and return it as unencoded bytes"""
-        return self.obj.read_bytes()
+        return self.obj.read_bytes(decode_level=StreamDecodeLevel.specialized)
 
     def get_stream_buffer(self):
         """Access this image with the buffer protocol"""
-        return self.obj.get_stream_buffer()
+        return self.obj.get_stream_buffer(decode_level=StreamDecodeLevel.specialized)
 
     def as_pil_image(self):
         """Extract the image as a Pillow Image, using decompression as necessary
@@ -533,7 +551,7 @@ class PdfImage(PdfImageBase):
             self._extract_direct(stream=bio)
             bio.seek(0)
             return Image.open(bio)
-        except UnsupportedImageTypeError:
+        except NotExtractableError:
             pass
 
         im = self._extract_transcoded()
@@ -550,12 +568,24 @@ class PdfImage(PdfImageBase):
         if not self.decode_parms:
             raise ValueError("/CCITTFaxDecode without /DecodeParms")
 
-        if self.decode_parms[0].get("/K", 1) < 0:
+        if self.decode_parms[0].get("/EncodedByteAlign", False):
+            raise UnsupportedImageTypeError(
+                "/CCITTFaxDecode with /EncodedByteAlign true"
+            )
+
+        k = self.decode_parms[0].get("/K", 0)
+        if k < 0:
             ccitt_group = 4  # Pure two-dimensional encoding (Group 4)
+        elif k > 0:
+            ccitt_group = 3  # Group 3 2-D
         else:
-            ccitt_group = 3
+            ccitt_group = 2  # CCITT 1-D
         black_is_one = self.decode_parms[0].get("/BlackIs1", False)
-        white_is_zero = 1 if black_is_one else 0
+        # TIFF spec says:
+        # use 0 for white_is_zero (=> black is 1)
+        # use 1 for black_is_zero (=> white is 1)
+        # In practice, do the opposite of what the TIFF spec says.
+        photometry = 1 if black_is_one else 0
 
         img_size = len(data)
         tiff_header_struct = '<' + '2s' + 'H' + 'L' + 'H' + 'HHLL' * 8 + 'L'
@@ -570,7 +600,7 @@ class PdfImage(PdfImageBase):
             257, 4, 1, self.height,  # ImageLength, LONG, 1, length
             258, 3, 1, 1,  # BitsPerSample, SHORT, 1, 1
             259, 3, 1, ccitt_group,  # Compression, SHORT, 1, 4 = CCITT Group 4 fax encoding
-            262, 3, 1, int(white_is_zero),  # Thresholding, SHORT, 1, 0 = WhiteIsZero
+            262, 3, 1, int(photometry),  # Thresholding, SHORT, 1, 0 = WhiteIsZero
             273, 4, 1, struct.calcsize(tiff_header_struct),  # StripOffsets, LONG, 1, length of header
             278, 4, 1, self.height,
             279, 4, 1, img_size,  # StripByteCounts, LONG, 1, size of image
